use std::any::{TypeId, type_name};
//* # See what's behind the primitive datatypes of Rust
//*
//* This Crate shows off, how primitive Types of rust are stored in memory.

fn main() {
    let mut items = Vec::new();
    items.push(true);
    items.push(false);
    investigate!(bool, items);

    let mut items = Vec::new();
    items.push(String::from("foo"));
    items.push(String::from("bar"));
    items.push(String::from("文学"));
    investigate!(String, items);
}

#[macro_export]
macro_rules! investigate {
    ($t:ty, $v:tt) => {
        println!("Type:\t{}", type_name::<$t>());
        println!("\tID:\t{:?}", TypeId::of::<$t>());
        println!("\tItems:");
        unsafe {
            for (index, item) in $v.iter().enumerate() {
                let pointer = item as *const $t;
                let raw_pointer: [u8; std::mem::size_of::<$t>()] = std::mem::transmute(item.clone());
                println!("\t{index:02x}\titem:\t{item:?}\n\
                    \t\tpointer: {:X?}\n\
                    \t\talign: {:#X} B\n\
                    \t\tmemory: {:X?}\n",
                    pointer,
                    std::mem::align_of::<$t>(),
                    raw_pointer,
                );
            }
        }
    };
}
